; Copyright 1998 Paulo Jose de Matos Lopes Pinto & Pablo Oliveira Tavares
.MODEL SMALL
.CODE
;--------------------------------------
;| Rotinas publicas a outros modulos. |
;--------------------------------------
PUBLIC IsDigit
PUBLIC IsLetter
PUBLIC IsOp
PUBLIC Atoi
PUBLIC Itoa
PUBLIC ToUpper


;------------------------------------------------------
;| Rotina: int IsDigit (char Digit)                   |
;------------------------------------------------------
;| Recebe Digit na stack.                             |
;| Verifica se Digit e um digito.                     |
;| Devolve em ax o valor. 1 -> Verdadeiro, 0 -> Falso.|
;------------------------------------------------------

; Para Simplificar a percepcao da rotina
Digit EQU WORD PTR [bp + 4]

IsDigit         PROC
                push   bp                 ; Guarda os valores na stack
                mov    bp, sp

    IF01:       mov    ax, Digit          ; Verifica a condicao 
                cmp    al, '0'            ; Digit >= '0' e Digit <= '9'
                jb     IF01_ELSE          
                cmp    al, '9'
                ja     IF01_ELSE
    IF01_THEN:  mov    ax, 1              ; Se sim ax = 1
                jmp    SHORT IF01_END
    IF01_ELSE:  mov    ax, 0              ; Se nao ax = 0
     
    IF01_END:   pop    bp                 ; Restaura os registos e elimina
                ret    2                  ; Digit da stack
IsDigit         ENDP


;------------------------------------------------------
;| Rotina: int IsLetter (char Letter)                 |
;------------------------------------------------------
;| Recebe Letter na stack.                            |
;| Verifica se Letter e uma letra.                    |
;| Devolve em ax o valor. 1 -> Verdadeiro, 0 -> Falso.|
;------------------------------------------------------

; Para Simplificar a percepcao da rotina
Letter EQU WORD PTR [bp + 4]

IsLetter        PROC
                push   bp                 ; Guarda os valores na stack
                mov    bp, sp

    IF02:       mov    ax, Letter         ; Verifica a condicao 
                cmp    al, 'A'            ; (Letter >= 'A' e Letter <= 'Z') ou
                jb     IF02_ELSE          ; (Letter >= 'a' e Letter <= 'z')
                cmp    al, 'Z'            
                jbe    IF02_THEN
                cmp    al, 'a'
                jb     IF02_ELSE
                cmp    al, 'z'
                ja     IF02_ELSE
    IF02_THEN:  mov    ax, 1              ; Se sim ax = 1
                jmp    SHORT IF02_END
    IF02_ELSE:  mov    ax, 0              ; Se nao ax = 0
     
    IF02_END:   pop    bp                 ; Restaura os registos e elimina
                ret    2                  ; Letter da stack
IsLetter        ENDP

;------------------------------------------------------
;| Rotina: int IsOp (char Op)                         |
;------------------------------------------------------
;| Recebe Op na stack.                                |
;| Verifica se OP e um dos operadores +,-,*,/,^.      |
;| Devolve em ax o valor. 1 -> Verdadeiro, 0 -> Falso.|
;------------------------------------------------------

; Para Simplificar a percepcao da rotina
Op EQU WORD PTR [bp + 4]

IsOp            PROC
                push   bp                 ; Guarda os valores na stack
                mov    bp, sp

    IF03:       mov    ax, Op             ; Verifica a condicao 
                cmp    al, '+'            ; Op = '+' ou Op = '-' ou Op = '*'
                je     IF03_THEN          ; ou Op = '/' ou Op = '^' ou
                cmp    al, '-'            ; Op = '(' ou Op = ')' ou Op = '='
                je     IF03_THEN
                cmp    al, '*'  
                je     IF03_THEN
                cmp    al, '/'  
                je     IF03_THEN
                cmp    al, '^'  
                je     IF03_THEN
                cmp    al, '('  
                je     IF03_THEN
                cmp    al, ')'  
                je     IF03_THEN
                cmp    al, '='  
                jne    IF03_ELSE
    IF03_THEN:  mov    ax, 1              ; Se sim ax = 1
                jmp    SHORT IF02_END
    IF03_ELSE:  mov    ax, 0              ; Se nao ax = 0
     
    IF03_END:   pop    bp                 ; Restaura os registos e elimina
                ret    2                  ; Op da stack
IsOp        ENDP

;------------------------------------------------------
;| Rotina: int Atoi (char *StartPos)                  |
;------------------------------------------------------
;| Recebe StartPos na stack.                          |
;| Converte uma string de numeros para inteiro.       |
;| Para quando o caracter nao for um digito.          |
;| Devolve em ax o valor, em bx o local onde parou e  |
;| em dx se conseguiu converter todo o numero ou nao, |
;| ou seja se dx = 1, conseguiu, se dx = 0 nao        |
;| conseguiu.                                         |
;------------------------------------------------------

; Para Simplificar a percepcao da rotina
StartPos   EQU WORD PTR [bp + 4]

Atoi            PROC
                push   bp                  ; Guarda os valores na stack
                mov    bp, sp
                push   cx

                
                xor    cx, cx              ; E mais rapido que mov cx, 0
                mov    bx, StartPos
    WHILE1:     mov    dx, 1
                xor    ah, ah              ; Ve se [Ptr] e um digito
                mov    al, BYTE PTR [bx]   
                push   ax                  
                call   IsDigit             
                cmp    ax, 1               ; Se sim adiciona-o a ax
                jne    WHILE1_END
                mov    ax, cx              ; Num := Num * 10 + ([Ptr] - '0')
                mov    cx, 10
                mul    cx               
    IF04:       jno    IF04_END            ; Se deu overflow e porque precisa
    IF04_THEN:  mov    dx, 0               ; de mais bits, o neste caso nao
                jmp    SHORT WHILE1_END    ; e possivel porque o nosso limite
    IF04_END:                              ; e de 16 bits
                mov    cl, BYTE PTR [bx]
                sub    cl, '0'
                xor    ch, ch
                add    cx, ax              ; E' feito assim para evitar realizar
                                           ; de seguida um mov cx, ax
                inc    bx                  ; E vai para o proximo caracter
                jmp    SHORT WHILE1
    WHILE1_END: mov    ax, cx

                pop    cx                  ; Restaura os registos e elimina
                pop    bp                  
                ret    2                   ; StartPos da stack
Atoi        ENDP

;------------------------------------------------------
;| Rotina: void Itoa (int Value, char *StartPos)      |
;------------------------------------------------------
;| Recebe StartPos na stack.                          |
;| Converte um inteiro para uma string de numeros.    |
;| O buffer tem que ter espaco suficiente para o      |
;| numero com o respectivo sinal.                     |
;| Os parametros sao colocados na stack da esquerda   |
;| para a direita.                                    |
;------------------------------------------------------

; Para Simplificar a percepcao da rotina
Value      EQU WORD PTR [bp + 6]
StartPos   EQU WORD PTR [bp + 4]

Itoa            PROC
                push   bp                  ; Guarda os valores na stack
                mov    bp, sp
                push   bx
                push   cx
                push   dx


    IF05:       mov    ax, Value           ; Verifica se Value < 0
                test   ax, 8000h
                jz     IF05_END
    IF05_THEN:  neg    ax                  ; Se for passa-o a positivo e     
                mov    bx, StartPos        ; coloca o sinal - na string
                mov    BYTE PTR [bx], '-'
                inc    StartPos
    IF05_END:   xor    cx, cx              ; Coloca cx a 0
                mov    bx, 10
    REPEAT1:    xor    dx, dx              ; Executa as divisoes sucessivas
                div    bx                  ; por 10 para converter para base
                push   dx                  ; 10
                inc    cx                  
                cmp    ax, 0
    UNTIL1:     jne    REPEAT1
                mov    bx, StartPos
    FOR1:       pop    ax                  ; Copia os caracteres para o 
                add    al, '0'             ; buffer
                mov    BYTE PTR [bx], al   
                inc    bx
    FOR1_END:   loop   FOR1
                mov    BYTE PTR [bx], 0    ; Termina a string com 0

                pop    dx
                pop    cx                  ; Restaura os registos e elimina
                pop    bx
                pop    bp                 
                ret    4                   ; Value e StartPos da stack
Itoa        ENDP

;------------------------------------------------------
;| Rotina: char ToUpper (char Letter)                 |
;------------------------------------------------------
;| Recebe Letter na stack.                            |
;| Passa a maiuscula a letra.                         |
;| Devolve em ax o valor.                             |
;------------------------------------------------------

; Para Simplificar a percepcao da rotina
Letter EQU WORD PTR [bp + 4]

ToUpper         PROC
                push   bp                 ; Guarda os valores na stack
                mov    bp, sp

    IF06:       mov    ax, Letter         ; Verifica a condicao 
                cmp    al, 'a'            ; (Letter >= 'a' e Letter <= 'z')
                jb     IF06_END           
                cmp    al, 'z'
                ja     IF06_END
    IF06_THEN:  sub    al, 'a' - 'A'      ; Passa para maiuscula
    
    IF06_END:   pop    bp                 ; Restaura os registos e elimina
                ret    2                  ; Letter da stack
ToUpper         ENDP


END
